import React from "react";
import { ICellElementCss, IColumn, IDataTableOptions, IHeaderElementCss, IPaginationCss } from "./interfaces";
export declare type TableColumn = IColumn & {
    isSorted: boolean;
    sortDirection?: Sorting;
};
export declare type HeaderItem = {
    isHeaderClicked: boolean;
    onSortClicked: (sortDirection: Sorting, headerItem: TableColumn, event: React.MouseEvent) => void;
    prevDirection: Sorting;
    isHovered?: boolean;
    props: HeaderProps;
};
export declare type SortOptions = {
    sortArray: any[];
    stortKey: string;
    sortDirection?: Sorting;
};
export declare type CellProps = {
    dense?: {
        isDense: boolean;
        denseCss: string;
    };
    classNames?: ICellElementCss;
    customCellStyle?: React.CSSProperties;
    children?: React.ReactNode;
};
export declare enum Sorting {
    ASC = 0,
    DESC = 1
}
export declare type HeaderProps = {
    item: TableColumn;
    sortState?: {
        sortKey: string | null | undefined;
        isSorted: boolean;
    };
    sortIcon?: React.FC;
    dense?: {
        isDense: boolean;
        denseCss: string;
    };
    classNames?: IHeaderElementCss;
    onHeaderClick?: (isSortOpen: boolean, selectedItem: TableColumn, event: React.MouseEvent<HTMLButtonElement>) => void;
    onSortIconClick?: (sortDirection: Sorting, headerItem: TableColumn, event: React.MouseEvent) => void;
};
export declare type RowProps = {
    header: Array<IColumn>;
    dense: {
        isDense: boolean;
        denseCss: string;
    };
    striped: {
        isStriped: boolean;
        stripedCss: string;
    };
    onHover: {
        isHoverRequired: boolean;
        onHoverCss: string;
    };
    classNames: {
        rowElementCss: string;
        cellElementCss: ICellElementCss;
    };
    dataItem: any;
    index: string;
    rowSingleClicked?: (row: any, event: React.MouseEvent<HTMLTableRowElement> | React.TouchEvent<HTMLTableRowElement>) => void;
    rowDoubleClicked?: (row: any, event: React.MouseEvent<HTMLTableRowElement> | React.TouchEvent<HTMLTableRowElement>) => void;
    customRowStyles?: RowStyleCustom[];
};
export declare type PaginationProps = {
    classNames: IPaginationCss;
    tableOptions: IDataTableOptions;
    data: any[];
    updateRowsPerPage: (data: any[], newIndex: number) => void;
};
export declare type DeepMerge = {
    target: any;
    source: any;
};
export declare type RowsPerPage = {
    option: number[];
    defaultIndex: number;
};
export declare type RowStyleCustom = {
    when: (row: any) => boolean;
    style: (row: any) => React.CSSProperties;
};
export declare type CellStyleCustom = {
    when: (value: any) => boolean;
    style: () => React.CSSProperties;
};
